@@ -6,15 +6,12 @@
 
 #include "application.h"
 #include "buildcfg.h"
-#include "event.h"
 #include "exif.h"
 #include "imagelist.h"
 
 #include <errno.h>
 #include <fcntl.h>
-#include <poll.h>
 #include <pthread.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
@@ -29,7 +26,7 @@
     enum loader_status LOADER_FUNCTION(name)(struct image * ctx, \
                                              const uint8_t* data, size_t size)
 
-const char* supported_formats = "bmp, pnm, tga"
+const char* supported_formats = "bmp, pnm, farbfeld, tga"
 #ifdef HAVE_LIBJPEG
                                 ", jpeg"
 #endif
@@ -62,11 +59,16 @@ const char* supported_formats = "bmp, pnm, tga"
 #endif
 #ifdef HAVE_LIBTIFF
                                 ", tiff"
+#endif
+#ifdef HAVE_LIBDICOM
+                                ", dicom"
 #endif
     ;
 
 // declaration of loaders
 LOADER_DECLARE(bmp);
+LOADER_DECLARE(dicom);
+LOADER_DECLARE(farbfeld);
 LOADER_DECLARE(pnm);
 LOADER_DECLARE(qoi);
 LOADER_DECLARE(tga);
@@ -101,6 +103,7 @@ LOADER_DECLARE(tiff);
 LOADER_DECLARE(webp);
 #endif
 
+// clang-format off
 // list of available decoders
 static const image_decoder decoders[] = {
 #ifdef HAVE_LIBJPEG
@@ -112,7 +115,9 @@ static const image_decoder decoders[] = {
 #ifdef HAVE_LIBGIF
     &LOADER_FUNCTION(gif),
 #endif
-    &LOADER_FUNCTION(bmp),  &LOADER_FUNCTION(pnm),
+    &LOADER_FUNCTION(bmp),
+    &LOADER_FUNCTION(pnm),
+    &LOADER_FUNCTION(dicom),
 #ifdef HAVE_LIBWEBP
     &LOADER_FUNCTION(webp),
 #endif
@@ -134,8 +139,11 @@ static const image_decoder decoders[] = {
 #ifdef HAVE_LIBTIFF
     &LOADER_FUNCTION(tiff),
 #endif
-    &LOADER_FUNCTION(qoi),  &LOADER_FUNCTION(tga),
+    &LOADER_FUNCTION(qoi),
+    &LOADER_FUNCTION(farbfeld),
+    &LOADER_FUNCTION(tga) // should be the last one
 };
+// clang-format on
 
 /** Background thread loader queue. */
 struct loader_queue {
