@@ -7,7 +7,9 @@
 #include "application.h"
 #include "buildcfg.h"
 #include "config.h"
-#include "xdg-shell-protocol.h"
+#include "content-type-v1-client-protocol.h"
+#include "cursor-shape-v1-client-protocol.h"
+#include "xdg-shell-client-protocol.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -50,6 +52,9 @@ struct ui {
         struct wl_keyboard* keyboard;
         struct wl_pointer* pointer;
         struct wl_surface* surface;
+        struct wp_cursor_shape_manager_v1* cursor_manager;
+        struct wp_content_type_manager_v1* content_type_manager;
+        struct wp_content_type_v1* content_type;
         struct wl_output* output;
     } wl;
 
@@ -243,6 +248,9 @@ static void on_keyboard_enter(void* data, struct wl_keyboard* wl_keyboard,
 static void on_keyboard_leave(void* data, struct wl_keyboard* wl_keyboard,
                               uint32_t serial, struct wl_surface* surface)
 {
+    // reset keyboard repeat timer
+    struct itimerspec ts = { 0 };
+    timerfd_settime(ctx.repeat.fd, 0, &ts, NULL);
 }
 
 static void on_keyboard_modifiers(void* data, struct wl_keyboard* wl_keyboard,
@@ -308,10 +316,30 @@ static void on_keyboard_key(void* data, struct wl_keyboard* wl_keyboard,
     }
 }
 
+/**
+ * Set mouse pointer shape.
+ * @param wl_pointer wayland pointer instance
+ * @param grabbing true to set grabbing shape, false to use default
+ */
+static void set_pointer_shape(struct wl_pointer* wl_pointer, bool grabbing)
+{
+    if (ctx.wl.cursor_manager) {
+        struct wp_cursor_shape_device_v1* cursor_device =
+            wp_cursor_shape_manager_v1_get_pointer(ctx.wl.cursor_manager,
+                                                   wl_pointer);
+        wp_cursor_shape_device_v1_set_shape(
+            cursor_device, 0,
+            grabbing ? WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_GRABBING
+                     : WP_CURSOR_SHAPE_DEVICE_V1_SHAPE_DEFAULT);
+        wp_cursor_shape_device_v1_destroy(cursor_device);
+    }
+}
+
 static void on_pointer_enter(void* data, struct wl_pointer* wl_pointer,
                              uint32_t serial, struct wl_surface* surface,
                              wl_fixed_t surface_x, wl_fixed_t surface_y)
 {
+    set_pointer_shape(wl_pointer, false);
 }
 
 static void on_pointer_leave(void* data, struct wl_pointer* wl_pointer,
@@ -346,6 +374,7 @@ static void on_pointer_button(void* data, struct wl_pointer* wl_pointer,
 
     if (button == BTN_LEFT) {
         ctx.mouse.active = pressed;
+        set_pointer_shape(wl_pointer, pressed);
     }
 
     if (pressed) {
@@ -597,6 +626,14 @@ static void on_registry_global(void* data, struct wl_registry* registry,
     } else if (strcmp(interface, wl_seat_interface.name) == 0) {
         ctx.wl.seat = wl_registry_bind(registry, name, &wl_seat_interface, 4);
         wl_seat_add_listener(ctx.wl.seat, &seat_listener, data);
+    } else if (strcmp(interface, wp_content_type_manager_v1_interface.name) ==
+               0) {
+        ctx.wl.content_type_manager = wl_registry_bind(
+            registry, name, &wp_content_type_manager_v1_interface, 1);
+    } else if (strcmp(interface, wp_cursor_shape_manager_v1_interface.name) ==
+               0) {
+        ctx.wl.cursor_manager = wl_registry_bind(
+            registry, name, &wp_cursor_shape_manager_v1_interface, 1);
     }
 }
 
@@ -676,6 +713,14 @@ bool ui_init(const char* app_id, size_t width, size_t height)
         xdg_toplevel_set_fullscreen(ctx.xdg.toplevel, NULL);
     }
 
+    if (ctx.wl.content_type_manager) {
+        ctx.wl.content_type =
+            wp_content_type_manager_v1_get_surface_content_type(
+                ctx.wl.content_type_manager, ctx.wl.surface);
+        wp_content_type_v1_set_content_type(ctx.wl.content_type,
+                                            WP_CONTENT_TYPE_V1_TYPE_PHOTO);
+    }
+
     wl_surface_commit(ctx.wl.surface);
 
     app_watch(wl_display_get_fd(ctx.wl.display), on_wayland_event, NULL);
@@ -732,6 +777,15 @@ void ui_destroy(void)
     if (ctx.wl.compositor) {
         wl_compositor_destroy(ctx.wl.compositor);
     }
+    if (ctx.wl.content_type) {
+        wp_content_type_v1_destroy(ctx.wl.content_type);
+    }
+    if (ctx.wl.content_type_manager) {
+        wp_content_type_manager_v1_destroy(ctx.wl.content_type_manager);
+    }
+    if (ctx.wl.cursor_manager) {
+        wp_cursor_shape_manager_v1_destroy(ctx.wl.cursor_manager);
+    }
     if (ctx.wl.registry) {
         wl_registry_destroy(ctx.wl.registry);
     }
@@ -809,6 +863,19 @@ void ui_set_title(const char* name)
     }
 }
 
+void ui_set_content_type_animated(bool animated)
+{
+    if (ctx.wl.content_type) {
+        if (animated) {
+            wp_content_type_v1_set_content_type(ctx.wl.content_type,
+                                                WP_CONTENT_TYPE_V1_TYPE_VIDEO);
+        } else {
+            wp_content_type_v1_set_content_type(ctx.wl.content_type,
+                                                WP_CONTENT_TYPE_V1_TYPE_PHOTO);
+        }
+    }
+}
+
 size_t ui_get_width(void)
 {
     return ctx.wnd.width;
