@@ -9,12 +9,9 @@
 #include "fetcher.h"
 #include "imagelist.h"
 #include "info.h"
-#include "keybind.h"
-#include "loader.h"
 #include "ui.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <sys/timerfd.h>
 #include <unistd.h>
@@ -30,6 +27,7 @@
 #define CFG_WINDOW_DEF         ARGB(0, 0, 0, 0)
 #define CFG_TRANSPARENCY_DEF   GRID_NAME
 #define CFG_SCALE_DEF          "optimal"
+#define CFG_POSITION_DEF       "center"
 #define CFG_FIXED_DEF          true
 #define CFG_ANTIALIASING_DEF   false
 #define CFG_SLIDESHOW_DEF      false
@@ -62,6 +60,32 @@ static const char* scale_names[] = {
 };
 // clang-format on
 
+enum position {
+    position_top,
+    position_center,
+    position_bottom,
+    position_left,
+    position_right,
+    position_top_left,
+    position_top_right,
+    position_bottom_left,
+    position_bottom_right,
+};
+
+// clang-format off
+static const char* position_names[] = {
+    [position_top] = "top",
+    [position_center] = "center",
+    [position_bottom] = "bottom",
+    [position_left] = "left",
+    [position_right] = "right",
+    [position_top_left] = "topleft",
+    [position_top_right] = "topright",
+    [position_bottom_left] = "bottomleft",
+    [position_bottom_right] = "bottomright",
+};
+// clang-format on
+
 /** Viewer context. */
 struct viewer {
     ssize_t img_x, img_y; ///< Top left corner of the image
@@ -72,6 +96,7 @@ struct viewer {
     bool fixed;           ///< Fix image position
 
     enum fixed_scale scale_init; ///< Initial scale
+    enum position position;      ///< Initial position
     float scale;                 ///< Current scale factor of the image
 
     bool animation_enable; ///< Animation enable/disable
@@ -231,13 +256,47 @@ static void scale_image(enum fixed_scale sc)
             break;
     }
 
-    // center viewport
-    ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
-    ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+    switch (ctx.position) {
+        case position_top:
+            ctx.img_y = 0;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_center:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_bottom:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_left:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = 0;
+            break;
+        case position_right:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+        case position_top_left:
+            ctx.img_y = 0;
+            ctx.img_x = 0;
+            break;
+        case position_top_right:
+            ctx.img_y = 0;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+        case position_bottom_left:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = 0;
+            break;
+        case position_bottom_right:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+    }
 
     fixup_position(true);
     info_update(info_scale, "%.0f%%", ctx.scale * 100);
-    app_redraw();
 }
 
 /**
@@ -295,6 +354,34 @@ static void zoom_image(const char* params)
     app_redraw();
 }
 
+/**
+ * Set default scale to a fixed scale value and apply it.
+ * @param params fixed scale to set
+ */
+static void scale_global(const char* params)
+{
+    if (params && *params) {
+        ssize_t fixed_scale = str_index(scale_names, params, 0);
+
+        if (fixed_scale >= 0) {
+            ctx.scale_init = fixed_scale;
+        } else {
+            fprintf(stderr, "Invalid scale operation: \"%s\"\n", params);
+            return;
+        }
+    } else {
+        // toggle to the next scale
+        ctx.scale_init++;
+        if (ctx.scale_init >= ARRAY_SIZE(scale_names)) {
+            ctx.scale_init = 0;
+        }
+    }
+
+    info_update(info_status, "Scale %s", scale_names[ctx.scale_init]);
+    scale_image(ctx.scale_init);
+    app_redraw();
+}
+
 /**
  * Start/stop animation if image supports it.
  * @param enable state to set
@@ -346,7 +433,6 @@ static void reset_state(void)
     ctx.img_y = 0;
     ctx.scale = 0;
     scale_image(ctx.scale_init);
-    fixup_position(true);
 
     ui_set_title(img->name);
     animation_ctl(true);
@@ -357,6 +443,8 @@ static void reset_state(void)
         info_update(info_index, "%zu of %zu", img->index + 1, total_img);
     }
 
+    ui_set_content_type_animated(ctx.animation_enable);
+
     app_redraw();
 }
 
@@ -410,6 +498,9 @@ static bool next_image(enum action_type direction)
             case action_next_file:
                 index = image_list_next_file(index);
                 break;
+            case action_rand_file:
+                index = image_list_rand_file(index);
+                break;
             default:
                 break;
         }
@@ -564,6 +655,7 @@ static void apply_action(const struct action* action)
         case action_next_dir:
         case action_prev_file:
         case action_next_file:
+        case action_rand_file:
             next_image(action->type);
             break;
         case action_skip_file:
@@ -604,6 +696,9 @@ static void apply_action(const struct action* action)
         case action_zoom:
             zoom_image(action->params);
             break;
+        case action_scale:
+            scale_global(action->params);
+            break;
         case action_rotate_left:
             rotate_image(false);
             break;
@@ -687,6 +782,15 @@ void viewer_init(struct config* cfg, struct image* image)
         config_error_val(VIEWER_SECTION, value);
     }
 
+    value = config_get_string(cfg, VIEWER_SECTION, VIEWER_POSITION,
+                              CFG_POSITION_DEF);
+    index = str_index(position_names, value, 0);
+    if (index >= 0) {
+        ctx.position = index;
+    } else {
+        ctx.position = position_center;
+    }
+
     // cache and preloads
     history = config_get_num(cfg, VIEWER_SECTION, VIEWER_HISTORY, 0, 1024,
                              CFG_HISTORY_DEF);
