@@ -57,6 +57,8 @@ TEST_F(Loader, External)
     }
 
 TEST_LOADER(bmp);
+TEST_LOADER(dcm);
+TEST_LOADER(ff);
 TEST_LOADER(pnm);
 TEST_LOADER(qoi);
 TEST_LOADER(tga);
